/*=============================================================================
vitalnix/daxtraq - User Management UI
ds_sdf.c - SDF data source module
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 1999 - 2003
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
/* This module is documented in some places. More information can be found in
doc/ds_mod.html and doc/ds_sdf.html, including the SDF file format specs. */
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libHX.h>
#include "dxt/ds.h"
#define MAX_FIELDS 5
#define MAXLNLEN 1024 /* max length for line buffer */

typedef unsigned char byte;

MODULE_NAME("SDF Data Source module");
MODULE_DESC("no description");
MODULE_TYPE("DS");

struct state { FILE *fp; };

int q_open(char *, void **);
int q_read(void *, struct ds_entry *);
void q_close(void *);
int q_sysctl(void *, ...);

static signed char only_digits(char *);

//-----------------------------------------------------------------------------
int q_open(char *fn, void **state) {
  struct state *sp;
  char buf[MAXLNLEN];
  int eax;
  if((sp = *state = malloc(sizeof(struct state))) == NULL) { return -errno; }
  if((sp->fp = fopen(fn, "r")) == NULL) {
    eax = -errno;
    free(sp);
    return eax;
  }

  while(fgets(buf, MAXLNLEN, sp->fp) != NULL) {
    char *bufp = buf, *field[MAX_FIELDS], **field_travp = field;
    size_t fields = 0;
    HX_chomp(buf);
    if(*buf == '#') { continue; }
    if(buf[0] == '(' && buf[1] == '*') {
      int s = strlen(buf) - 1;
      if(buf[s - 1] == '*' && buf[s] == ')') { continue; }
    }

    while(strlen(bufp) > 0 && fields <= MAX_FIELDS) {
      *field_travp++ = bufp;
      ++fields;
      if((bufp = strchr(bufp, ';')) != NULL) { *bufp++ = '\0'; }
      else { break; }
    }

    if(fields >= 4) {
      fseek(sp->fp, 0, SEEK_SET);
      return 1;
    }
  }

  fclose(sp->fp);
  return 0;
}

int q_read(void *state, struct ds_entry *x) {
  struct state *sp = state;
  char buf[MAXLNLEN];
  while(fgets(buf, MAXLNLEN, sp->fp) != NULL) {
    char *bufp = buf, *field[MAX_FIELDS], **field_travp = field, translat = 0;
    size_t count = 0;

    if(strrchr(buf, '\r') != NULL) { ++translat; }
    HX_chomp(buf);

    if(*buf == '#') { continue; }
    if(buf[0] == '(' && buf[1] == '*') {
      int s = strlen(buf) - 1;
      if(buf[s - 1] != '*' || buf[s] != ')') {
        fprintf(stderr, "Malformed comment in SDF file\n");
      }
      continue;
    }

    while(strlen(bufp) > 0 && count <= MAX_FIELDS) {
      *field_travp++ = bufp;
      ++count;
      if((bufp = strchr(bufp, ';')) != NULL) { *bufp++ = '\0'; }
      else { break; }
    }

    field_travp = field;
    if(only_digits(field[0])) { ++field_travp; }

    if(strlen(*field_travp) == 0) {
      x->nname = NULL;
    } else {
      x->nname = HX_strdup(*field_travp);
      if(translat) { HXcrypt_deascii(x->nname); }
    }
    ++field_travp;

    x->vname = HX_strdup(*field_travp++);
    if(translat) { HXcrypt_deascii(x->vname); }

    if(strlen(*field_travp) == 0) { x->xuid = NULL; }
    else { x->xuid = HX_strdup(*field_travp); }
    ++field_travp;

    if(strlen(*field_travp) == 0) { x->sgroup = NULL; }
    else { x->sgroup = HX_strdup(*field_travp); }
    return 1;
  }

  return 0; // eof
}

void q_close(void *state) {
  struct state *sp = state;
  fclose(sp->fp);
  free(sp);
  return;
}

//-----------------------------------------------------------------------------
static signed char only_digits(char *p) {
  while(*p) {
    if(!isdigit(*p)) { return 0; }
    ++p;
  }
  return 1;
}

//==[ End of file ]============================================================
