/*=============================================================================
vitalnix/accdb - User Management Suite - Unified Account Database
accdb.h - Include file for ACCDB back-end mod.s or applications utilizing them
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2003
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
#ifndef _ACCDB_H
#define _ACCDB_H 1

#ifdef __cplusplus
extern "C" {
#endif

struct accdb_user {
  // passwd part
  char *lname;
  long uid, gid;
  char *group, *gecos, *home, *shell;

  // shadow part
  char *pass;
  long last_change, keep_min, keep_max, warn_age, expiry, disabled_since;
  void *_private;
};

struct accdb_group {
  long gid;
  char *gname;
  /// XXX gshadow part??
  void *_private;
};

struct accdb_module {
  char *name, *desc, *path;
  void *handle, *state;
  int (*open)(void **);
  int (*usertrav)(void *, struct accdb_user *);
  int (*userinfo)(void *, struct accdb_user *, struct accdb_user *);
  int (*useradd)(void *, struct accdb_user *);
  int (*usermod)(void *, struct accdb_user *, struct accdb_user *);
  int (*userdel)(void *, struct accdb_user *);
  int (*grouptrav)(void *, struct accdb_group *);
  int (*groupinfo)(void *, struct accdb_group *, struct accdb_group *);
  int (*groupadd)(void *, struct accdb_group *);
  int (*groupmod)(void *, struct accdb_group *, struct accdb_group *);
  int (*groupdel)(void *, struct accdb_group *);
  int (*close)(void *);
  int (*sysctl)(unsigned int, ...);
};

struct rconfig_opt {
  const char *key;
  char type;
  void *ptr;
};

enum accdb_genpw_meth {
  PW_DES = 0,
  PW_MD5,
  PW_BLOWFISH,
};

enum accdb_genpw_flags {
  PW_PHONEMIC  = 1 << 0,
  PW_ONE_DIGIT = 1 << 1,
  PW_ONE_CASE  = 1 << 2,
};

enum accdb_rconfig_flags {
  RCONFIG_ONE  = 1 << 0, // only read one configuration file
  RCONFIG_BACK = 1 << 1, // read path vector backwards
};

enum accdb_sysctl_flags {
  ACCDB_NOAUTOFLUSH = 1 << 0,
};

enum accdb_sysctl_req {
  ACCDB_ADDFLAGS = 1,
  ACCDB_DELFLAGS,
  ACCDB_FLUSHDB,
  ACCDB_NEXTUID_SYS,
  ACCDB_NEXTUID,
  ACCDB_NEXTGID_SYS,
  ACCDB_NEXTGID,
  ACCDB_LOCK,
  ACCDB_UNLOCK,
  // module specific sysctl requests shall start at 0x4000
};

struct accdb_module *accdb_load(char *);
void accdb_unload(struct accdb_module *);
int accdb_genpw(char *, char **, size_t, enum accdb_genpw_meth,
 enum accdb_genpw_flags, void *);
extern int accdb_rconfig(char *, struct rconfig_opt *);
extern void accdb_rconfig_pv(const char **, char *, struct rconfig_opt *,
 enum accdb_rconfig_flags);
extern void accdb_rconfig_free(struct rconfig_opt *);

#ifdef __cplusplus
} // extern "C"
#endif

#endif /* _ACCDB_H */

//==[ End of file ]============================================================
